package lu.uni.mobile.bonk;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.util.ArrayList;

import org.json.JSONObject;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;

public class BackendService extends Service {
    private NotificationManager nm;
    private static boolean isRunning = false;
    private Game playground;
    private DatagramSocket datagramSocket;

    // all currently registered clients
    ArrayList<Messenger> clientMessengers = new ArrayList<Messenger>();

    int mValue = 0; // Holds last value set by a client.
    static final int MSG_TYPE_REGISTER_CLIENT = 1;
    static final int MSG_TYPE_UNREGISTER_CLIENT = 2;
    static final int MSG_TYPE_INVITATION = 3;
    static final int MSG_TYPE_ACCEPTANCE = 4;
    static final int MSG_TYPE_PLAYGROUND_SETUP = 5;
    static final int MSG_TYPE_GAME_DATA = 6;

    static final String MSG_LABEL_INVITATION = "invitation";
    static final String MSG_LABEL_ACCEPTANCE = "acceptance";
    static final String MSG_LABEL_PLAYGROUND_SETUP = "playgroundData";
    static final String MSG_LABEL_GAME_DATA = "gameData";

    // clients can send messages to IncomingHandler.
    final Messenger mMessenger = new Messenger(new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
            case MSG_TYPE_REGISTER_CLIENT:
                clientMessengers.add(msg.replyTo);
                break;
            case MSG_TYPE_UNREGISTER_CLIENT:
                clientMessengers.remove(msg.replyTo);
                break;
            default:
                super.handleMessage(msg);
            }
        }
    });

    @Override
    public void onCreate() {
        super.onCreate();
        Log.i("MyService", "Service Started.");
        // showStartupNotification();
        isRunning = true;
    }

    @Override
    public IBinder onBind(Intent intent) {
        return mMessenger.getBinder();
    }

    @SuppressWarnings("deprecation")
    private void showStartupNotification() {
        nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        CharSequence text = getText(R.string.serviceStarted); // Set the icon, scrolling text and timestamp
        Notification notification = new Notification(R.drawable.ic_launcher, text, System.currentTimeMillis());
        PendingIntent contentIntent = PendingIntent.getActivity(this, 0, new Intent(
            this,
            StartupActivity.class), 0); // Set the info for the views that show in the notification panel.
        notification.setLatestEventInfo(this, getText(R.string.serviceLabel), text, contentIntent); // Send
                                                                                                    // the
                                                                                                    // notification.
        // We use a layout id because it is a unique number. We use it later to cancel.
        nm.notify(R.string.serviceStarted, notification);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.i("MyService", "Received start id " + startId + ": " + intent);
        return START_STICKY; // run until explicitly stopped.
    }

    public static boolean isRunning() {
        return isRunning;
    }

    private void sendSimpleMessageToListeners(int messageType, int arg1, int arg2) {
        for (Messenger client : clientMessengers) {
            try {
                client.send(Message.obtain(null, messageType, arg1, arg2));
            } catch (RemoteException e) {
                clientMessengers.remove(client);
            }
        }
    }

    private void sendKeyValueMessageToListeners(int messageType, String key, String value) {
        for (Messenger client : clientMessengers) {
            try {
                Bundle b = new Bundle();
                b.putString(key, value);
                Message msg = Message.obtain(null, messageType);
                msg.setData(b);
                client.send(msg);
            } catch (RemoteException e) {
                clientMessengers.remove(client);
            }
        }
    }

    private void sendUdpJsonString(JSONObject jsonObject) {
        String dataString = jsonObject.toString();
        sendUdpByteString(dataString.getBytes(), dataString.length());
    }

    private void sendUdpByteString(byte[] data, int length) {
        DatagramPacket datagramPacket = new DatagramPacket(data, length);
        try {
            datagramSocket.send(datagramPacket);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        nm.cancel(R.string.serviceStarted); // Cancel the persistent notification.
        Log.i("MyService", "Service Stopped.");
        isRunning = false;
    }
}
