package lu.uni.mobile.bonk;

import java.net.InetAddress;
import java.net.UnknownHostException;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

public abstract class BonkActivity extends Activity {
    protected int userNumber;
    protected String[] userNames = new String[] { "Luke (fixed string)" };
    protected NetworkSender networkSender;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Intent intent = getIntent();
        userNumber = intent.getIntExtra("userNumber", 1);
        if (intent.hasExtra("userNames")) {
            userNames = intent.getStringArrayExtra("userNames");
        }

        NetworkListener.registerHandler(handler);

        networkSender = new NetworkSender();
        networkSender.start();
    }

    /**
     * Handler that the NetworkCommunicator (i.e. the I/O thread) will use to communicate updates from other
     * devices
     */
    public final Handler handler = new Handler() {
        @Override
        public void handleMessage(Message message) {
            Bundle msgData = message.getData();

            if (message.what == NetworkCommunicator.MSG_TYPE_INCOMING) {
                byte[] sender = msgData.getByteArray("sender");
                InetAddress senderAddress;
                try {
                    senderAddress = InetAddress.getByAddress(sender);
                } catch (UnknownHostException e) {
                    e.printStackTrace();
                    return;
                }
                byte[] data = msgData.getByteArray("data");
                int length = msgData.getInt("length");

                // first, try to decode a JSON string
                try {
                    String jsonString = new String(data, 0, length);
                    JSONObject json = new JSONObject(jsonString);
                    String jsonMessageType = json.getString(NetworkCommunicator.MSG_TYPE_JSON_KEY);
                    if (jsonMessageType.equals(NetworkCommunicator.MSG_TYPE_ACCEPTANCE)) {
                        handleAcceptanceJson(senderAddress, json);
                    }
                    else if (jsonMessageType.equals(NetworkCommunicator.MSG_TYPE_INVITATION)) {
                        handleInvitationJson(senderAddress, json);
                    }
                    else if (jsonMessageType.equals(NetworkCommunicator.MSG_TYPE_PLAYGROUND_SETUP)) {
                        handlePlaygroundSetupJson(senderAddress, json);
                    }
                    else {
                        handleUnknownJson(senderAddress, json);
                    }
                } catch (JSONException e) {
                    // if that fails, see if it's an object tracking/impact message (many messages will be
                    // concatenated together)
                    Log.i("NetworkCommunicator", this.getClass().getSimpleName()
                            + " got object tracking message: " + new String(data, 0, length));
                    handleGameUpdateMessage(senderAddress, data, length);
                }
            }
        }
    };

    /**
     * Handles the message sent from the (master) device of a player who has invited other players to join a
     * game.
     */
    protected void handleInvitationJson(InetAddress senderAddress, JSONObject json) {
    }

    /**
     * Handles the message sent from the (slave) device of a player who has accepted an invitation to join a
     */
    protected void handleAcceptanceJson(InetAddress senderAddress, JSONObject jsonDataFromMessage) {
    }

    /**
     * Handles the message to create the initial playground data
     */
    protected void handlePlaygroundSetupJson(InetAddress senderAddress, JSONObject json) {
    }

    /**
     * A JSON message whose messageType field is not understood
     */
    protected void handleUnknownJson(InetAddress senderAddress, JSONObject json) {
    }

    /**
     * handles the entire string of object tracking and/or impact messages sent in a game update
     */
    protected void handleGameUpdateMessage(InetAddress senderAddress, byte[] byteArray, int length) {
    }

    /**
     * 
     * A general helper function to wire a button to launch an activity
     */
    protected void setButtonActivityListener(int buttonId, final Class<? extends Activity> activityClass) {
        setButtonActivityListener(buttonId, activityClass, userNumber);// default to the current user number
    }

    /**
     * 
     * A general helper function to wire a button to launch an activity
     */
    protected void setButtonActivityListener(int buttonId, final Class<? extends Activity> activityClass,
            final int userNumber) {
        Button button = (Button) findViewById(buttonId);
        setButtonActivityListener(button, activityClass, userNumber);
    }

    /**
     * A general helper function to wire a button to launch an activity
     */
    protected void setButtonActivityListener(Button button, final Class<? extends Activity> activityClass,
            final int userNumber) {
        button.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                // NOTE: this bypasses the intent filter mechanism
                Intent intent = new Intent(BonkActivity.this, activityClass);
                intent.putExtra("userNames", userNames);
                intent.putExtra("userNumber", userNumber);
                startActivity(intent);
            }
        });
    }

    /**
     * a convenient wrapper function to send data via UDP broadcast/multicast to all other devices. The I/O is
     * done in the NetworkCommunicator thread to avoid blocking.
     */
    protected void sendUdpByteString(byte[] data, int length) {
        Message message = networkSender.getHandler().obtainMessage();
        message.what = NetworkCommunicator.MSG_TYPE_OUTGOING;
        Bundle bundle = new Bundle();
        bundle.putByteArray("data", data);
        bundle.putInt("length", length);
        message.setData(bundle);
        networkSender.getHandler().sendMessage(message);
    }
}