package lu.uni.mobile.bonk;

import java.io.IOException;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.InterfaceAddress;
import java.net.MulticastSocket;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.List;

import android.util.Log;

// TODO add TCP support
// TODO hard core Java geeks will say that subclassing Thread is bad form... 
public abstract class NetworkCommunicator extends Thread {
    public static final String MSG_TYPE_JSON_KEY = "messageType";
    public static final String MSG_TYPE_INVITATION = "invitation";
    public static final String MSG_TYPE_ACCEPTANCE = "acceptance";
    public static final String MSG_TYPE_PLAYGROUND_SETUP = "playgroundData";
    public static final String MSG_TYPE_GAME_DATA = "gameData";

    public static final int MSG_TYPE_OUTGOING = 0;
    public static final int MSG_TYPE_INCOMING = 1;

    public static final int MAX_PAYLOAD_BYTES = 512;

    // A multicast group is specified by a class D IP address and by a standard UDP port number. Class D IP
    // addresses are in the range 224.0.0.0 to 239.255.255.255, inclusive
    protected static final byte[] MULTICAST_GROUP = { (byte) 225, (byte) 11, (byte) 79, (byte) 163 };
    protected static final int MULTICAST_PORT = 7330;
    protected static final int COMMUNICATION_SLEEP_INTERVAL = 1000; // TODO should be about 50-100
    protected static final boolean IGNORE_MY_NETWORK_MESSAGES = false;

    protected NetworkInterface networkInterface = null; // Don't access this directly - use
    protected InetAddress myIpAddress;
    protected MulticastSocket multicastSocket;
    protected InetAddress multicastAddress;

    // getNetworkInterface() instead.

    public NetworkCommunicator() {
        try {
            multicastAddress = InetAddress.getByAddress(MULTICAST_GROUP);
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }
        try {
            multicastSocket = new MulticastSocket(MULTICAST_PORT);
            multicastSocket.joinGroup(multicastAddress);
        } catch (IOException e) {
            e.printStackTrace();
        }
        Log.i("NetworkCommunicator", "Network Communicator started.");
    }

    /**
     * Sets and returns myIpAddress
     */
    public InetAddress getMyIpAddress() {
        boolean interfaceNeedsRefreshing = true;
        try {
            interfaceNeedsRefreshing = !networkInterface.isUp();
        } catch (Exception e) {
        }

        if (myIpAddress == null || interfaceNeedsRefreshing) {
            List<InterfaceAddress> iAddresses = getNetworkInterface().getInterfaceAddresses();
            for (InterfaceAddress iAddress : iAddresses) {
                if (iAddress.getAddress() instanceof Inet4Address && iAddress.getBroadcast() != null) {
                    // found a usable V4 address. getBroadcast() is my way of assuming that there is a local
                    // subnet where other devices would show up.
                    // multicastAddress = broadcast; // I'm just saying we could if we wanted to.
                    myIpAddress = iAddress.getAddress();
                    break;
                }
            }
        }
        return myIpAddress;
    }

    /**
     * @return the first non-loopback interface found, or null if none qualify
     */
    protected NetworkInterface getNetworkInterface() {
        try {
            if (networkInterface == null || !networkInterface.isUp()) {
                Enumeration<NetworkInterface> interfaces = NetworkInterface.getNetworkInterfaces();
                while (interfaces.hasMoreElements()) {
                    NetworkInterface result = interfaces.nextElement();
                    if (!result.isLoopback() && result.supportsMulticast()) {
                        networkInterface = result;
                        break;
                    }
                }
                myIpAddress = null; // force it to refresh next time it's requested
            }
        } catch (SocketException e) {
        }
        return networkInterface;
    }

    @Override
    public String toString() {
        // for instance, "NetworkListener Thread [...]"
        return getClass().getSimpleName() + " " + super.toString();
    }

    public class PayloadOverLimitException extends Exception {
        private static final long serialVersionUID = 1L;
    }
}
