package lu.uni.mobile.bonk;

import java.net.DatagramPacket;
import java.util.HashSet;

import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class NetworkListener extends NetworkCommunicator {

    private static NetworkListener instance;
    private HashSet<Handler> listeners = new HashSet<Handler>();

    public static void registerHandler(Handler handler) {
        init();
        instance.listeners.add(handler);
    }

    public static void unRegisterHandler(Handler handler) {
        init();
        instance.listeners.remove(handler);
    }

    private static void init() {
        if (instance == null) {
            instance = new NetworkListener();
            instance.start();
        }
    }

    private NetworkListener() {
    }

    @Override
    public void run() {
        int i = 0;
        while (true) {
            try {
                int length = multicastSocket.getReceiveBufferSize();
                byte[] buffer = new byte[length];
                DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
                multicastSocket.receive(packet);

                onDataReceived(packet);

                Log.i("NetworkCommunicator", String.format("Received data %d times", ++i));
            } catch (Exception e) {
            }
            runLoopSleep();
        }
    }

    @SuppressWarnings("unused")
    protected void onDataReceived(DatagramPacket packet) {
        if (packet.getAddress().equals(getMyIpAddress()) && IGNORE_MY_NETWORK_MESSAGES) {
            return;
        }

        for (Handler handler : instance.listeners) {
            Message message = handler.obtainMessage();
            message.what = NetworkCommunicator.MSG_TYPE_INCOMING;
            Bundle bundle = new Bundle();
            bundle.putByteArray("sender", packet.getAddress().getAddress());
            bundle.putByteArray("data", packet.getData());
            bundle.putInt("length", packet.getLength());
            message.setData(bundle);
            handler.sendMessage(message);
        }
    };

    private void runLoopSleep() {
        try {
            sleep(COMMUNICATION_SLEEP_INTERVAL);
        } catch (InterruptedException e) {
        }
    }
}
