package lu.uni.mobile.bonk;

import java.io.IOException;
import java.net.DatagramPacket;

import org.json.JSONObject;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

public class NetworkSender extends NetworkCommunicator {

    /**
     * Handler that the BonkActivity (i.e. the UI) will use to send updates to the other players
     */
    private Handler handler;

    @Override
    public void run() {
        Looper.prepare();

        handler = new Handler() {
            public void handleMessage(Message msg) {
                Bundle msgData = msg.getData();

                if (msg.what == MSG_TYPE_OUTGOING) {
                    byte[] data = msgData.getByteArray("data");
                    int length = msgData.getInt("length");

                    try {
                        sendUdpByteString(data, length);
                        Log.i("NetworkCommunicator", "Sent UDP byte string via SenderThread");
                    } catch (PayloadOverLimitException e) {
                        e.printStackTrace();
                    }
                }
            }
        };

        Looper.loop();
    }

    public Handler getHandler() {
        return handler;
    }

    public void sendUdpJsonString(JSONObject jsonObject) throws PayloadOverLimitException {
        String dataString = jsonObject.toString();
        sendUdpByteString(dataString.getBytes(), dataString.length());
    }

    public void sendUdpByteString(byte[] data, int length) throws PayloadOverLimitException {
        DatagramPacket datagramPacket = new DatagramPacket(data, length, multicastAddress, MULTICAST_PORT);
        // TODO split message into multiple packets?
        // if (length > MAX_PAYLOAD_BYTES) {
        // throw new PayloadOverLimitException();
        // }
        try {
            multicastSocket.send(datagramPacket);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
