package lu.uni.mobile.bonk;

import java.net.InetAddress;
import java.util.Iterator;
import org.json.JSONException;
import org.json.JSONObject;

import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

public class PlayGameActivity extends BonkActivity {

    public static final String BUNDLE_KEY_TIMER_OFFSET = "timerOffset";
    public static final String BUNDLE_KEY_BLOCK_PLACING_TIME = "blockPlacingTime";
    public static final String BUNDLE_KEY_BLOCKS_PER_USER = "blocksPerUser";

    private static final float ROTATE_CW_AMOUNT = 25F;
    private static final float ROTATE_CCW_AMOUNT = -25F;
    private static final int DEFAULT_BLOCKS_PER_USER = 2;
    private static final int DEFAULT_BLOCK_PLACING_TIME_MS = 7000;
    private Game game;
    private boolean playgroundInitialized = false;
    private Game debugGame;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.play_game);

        ((Button) findViewById(R.id.btnFire)).setOnClickListener(fireButtonClick);
        ((Button) findViewById(R.id.btnCounterClockwise)).setOnClickListener(rotateCounterClockwise);
        ((Button) findViewById(R.id.btnClockwise)).setOnClickListener(rotateClockwise);

        game = new Game(
            this,
            (PlaygroundView) findViewById(R.id.playground),
            userNumber,
            getNumberOfBlocksPerUser(savedInstanceState),
            getGameStartTime(savedInstanceState));

        DO_DEBUG_SETUP(savedInstanceState);
    }

    private void DO_DEBUG_SETUP(Bundle savedInstanceState) {
        debugGame = new Game(
            this,
            (PlaygroundView) findViewById(R.id.playgroundNetworkView),
            userNumber,
            getNumberOfBlocksPerUser(savedInstanceState),
            getGameStartTime(savedInstanceState));
    }

    @Override
    protected void onStart() {
        // this fires after onCreate, so it's useful for drawing the game (because everything is initialized
        // now)
        super.onStart();
        game.createRandomGame();
    }

    protected int getNumberOfBlocksPerUser(Bundle savedInstanceState) {
        try {
            return savedInstanceState.getInt(BUNDLE_KEY_BLOCKS_PER_USER);
        } catch (Exception e) {
            return DEFAULT_BLOCKS_PER_USER;
        }
    }

    private long getGameStartTime(Bundle savedInstanceState) {
        // combines the current time, timer offset, and block-placing duration into a game start time in MS
        long timerOffset = 0;
        try {
            timerOffset = savedInstanceState.getLong(BUNDLE_KEY_TIMER_OFFSET);
        } catch (Exception e) {
            // keep the default
        }

        int blockPlacingTime = DEFAULT_BLOCK_PLACING_TIME_MS;
        try {
            blockPlacingTime = savedInstanceState.getInt(BUNDLE_KEY_BLOCK_PLACING_TIME);
        } catch (Exception e) {
            // keep the default
        }

        long gameStartTime = System.currentTimeMillis() + timerOffset + blockPlacingTime;
        return gameStartTime;
    }

    private OnClickListener fireButtonClick = new OnClickListener() {
        @Override
        public void onClick(View v) {
            game.fireBullet();
        }
    };

    private OnClickListener rotateClockwise = new OnClickListener() {
        @Override
        public void onClick(View v) {
            game.rotatePlayer(ROTATE_CW_AMOUNT);
        }
    };

    private OnClickListener rotateCounterClockwise = new OnClickListener() {
        @Override
        public void onClick(View v) {
            game.rotatePlayer(ROTATE_CCW_AMOUNT);
        }
    };

    @Override
    protected void handlePlaygroundSetupJson(InetAddress senderAddress, JSONObject json) {
        /**
         * { “target”: {“x”: int, “y”: int}, “players”: { “id 1”: { // a 3-bit int prefix for Object IDs
         * “name”: str, “ipAddress”: str, “x”: int, “y”: int, “blocks”: { “id 1”: { // a 13-bit Object ID
         * suffix “x”: int, “y”: int, }, “id 2”: {...} } }, “id 2”: { ... } } }
         */
        try {
            JSONObject targetJSON = json.getJSONObject("target");
            byte x = (byte) targetJSON.getInt("x");
            byte y = (byte) targetJSON.getInt("y");
            game.initTarget(x, y);
            debugGame.initTarget(x, y);

            JSONObject players = json.getJSONObject("players");
            @SuppressWarnings("unchecked")
            Iterator<String> playerIds = players.keys();
            while (playerIds.hasNext()) {
                String playerId = playerIds.next();
                JSONObject player = players.getJSONObject(playerId);
                String playerName = player.getString("name");
                String playerIpAddress = player.getString("ipAddress");
                x = (byte) player.getInt("x");
                y = (byte) player.getInt("y");
                game.initPlayer(Integer.parseInt(playerId), x, y, playerName, playerIpAddress);
                debugGame.initPlayer(Integer.parseInt(playerId), x, y, playerName, playerIpAddress);

                JSONObject blocks = player.getJSONObject("blocks");
                @SuppressWarnings("unchecked")
                Iterator<String> blockIds = blocks.keys();
                while (blockIds.hasNext()) {
                    String blockId = blockIds.next();
                    JSONObject block = blocks.getJSONObject(blockId);
                    x = (byte) block.getInt("x");
                    y = (byte) block.getInt("y");
                    game.initBlock(Integer.parseInt(blockId), x, y);
                    debugGame.initBlock(Integer.parseInt(blockId), x, y);
                }
            }
        } catch (JSONException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Override
    protected void handleGameUpdateMessage(InetAddress senderAddress, byte[] data, int length) {
        int timeStamp = data[0] << 8 | data[1];
        for (int i = 2; i < length;) {
            if (data[i] == Game.IMPACT_MESSAGE_INDICATOR_BYTE) {
                i++;
                int objectId1 = data[i++] << 8 | data[i++];
                int objectId2 = data[i++] << 8 | data[i++];
                // DEBUG
                // game.handleImpactMessage(timeStamp, objectId1, objectId2);
                debugGame.handleImpactMessage(timeStamp, objectId1, objectId2);
            }
            else {
                int objectId = data[i++] << 8 | data[i++];
                byte x = data[i++];
                byte y = data[i++];
                byte direction = data[i++];
                // DEBUG
                // game.handleObjectTrackingMessage(timeStamp, objectId, x, y, direction);
                debugGame.handleObjectTrackingMessage(timeStamp, objectId, x, y, direction);
            }
        }
    }
}
